@@ -1,9 +1,14 @@
-from neomodel import (StructuredNode, StringProperty, IntegerProperty, RelationshipFrom,
-        RelationshipTo, StructuredRel, DateTimeProperty)
-from neomodel.match import NodeSet, QueryBuilder
-from neomodel.exception import MultipleNodesReturned
 from datetime import datetime
 
+from pytest import raises
+
+from neomodel import (
+    INCOMING, DateTimeProperty, IntegerProperty, RelationshipFrom, RelationshipTo,
+    StringProperty, StructuredNode, StructuredRel, Q
+)
+from neomodel.match import NodeSet, QueryBuilder, Traversal
+from neomodel.exceptions import MultipleNodesReturned
+
 
 class SupplierRel(StructuredRel):
     since = DateTimeProperty(default=datetime.now)
@@ -20,6 +25,7 @@ class Coffee(StructuredNode):
     name = StringProperty(unique_index=True)
     price = IntegerProperty()
     suppliers = RelationshipFrom(Supplier, 'COFFEE SUPPLIERS', model=SupplierRel)
+    id_ = IntegerProperty()
 
 
 def test_filter_exclude_via_labels():
@@ -72,21 +78,13 @@ def test_get():
     Coffee(name='1', price=3).save()
     assert Coffee.nodes.get(name='1')
 
-    try:
+    with raises(Coffee.DoesNotExist):
         Coffee.nodes.get(name='2')
-    except Coffee.DoesNotExist:
-        assert True
-    else:
-        assert False
 
     Coffee(name='2', price=3).save()
 
-    try:
+    with raises(MultipleNodesReturned):
         Coffee.nodes.get(price=3)
-    except MultipleNodesReturned:
-        assert True
-    else:
-        assert False
 
 
 def test_simple_traverse_with_filter():
@@ -98,7 +96,7 @@ def test_simple_traverse_with_filter():
 
     results = qb.build_ast()._execute()
 
-    assert 'start' in qb._ast
+    assert 'lookup' in qb._ast
     assert 'match' in qb._ast
     assert qb._ast['return'] == 'suppliers'
     assert len(results) == 1
@@ -176,20 +174,12 @@ def test_contains():
     assert asda not in Coffee.nodes.filter(price__gt=999)
 
     # bad value raises
-    try:
+    with raises(ValueError):
         2 in Coffee.nodes
-    except ValueError:
-        assert True
-    else:
-        assert False
 
     # unsaved
-    try:
+    with raises(ValueError):
         Coffee() in Coffee.nodes
-    except ValueError:
-        assert True
-    else:
-        assert False
 
 
 def test_order_by():
@@ -231,10 +221,10 @@ def test_extra_filters():
     for c in Coffee.nodes:
         c.delete()
 
-    c1 = Coffee(name="Icelands finest", price=5).save()
-    c2 = Coffee(name="Britains finest", price=10).save()
-    c3 = Coffee(name="Japans finest", price=35).save()
-    c4 = Coffee(name="US extra-fine", price=None).save()
+    c1 = Coffee(name="Icelands finest", price=5, id_=1).save()
+    c2 = Coffee(name="Britains finest", price=10, id_=2).save()
+    c3 = Coffee(name="Japans finest", price=35, id_=3).save()
+    c4 = Coffee(name="US extra-fine", price=None, id_=4).save()
 
     coffees_5_10 = Coffee.nodes.filter(price__in=[10, 5]).all()
     assert len(coffees_5_10) == 2, "unexpected number of results"
@@ -250,3 +240,124 @@ def test_extra_filters():
     unpriced_coffees = Coffee.nodes.filter(price__isnull=True).all()
     assert len(unpriced_coffees) == 1, "unexpected number of results"
     assert c4 in unpriced_coffees, "doesnt contain unpriced coffee"
+
+    coffees_with_id_gte_3 = Coffee.nodes.filter(id___gte=3).all()
+    assert len(coffees_with_id_gte_3) == 2, "unexpected number of results"
+    assert c3 in coffees_with_id_gte_3
+    assert c4 in coffees_with_id_gte_3
+
+
+def test_traversal_definition_keys_are_valid():
+    muckefuck = Coffee(name='Mukkefuck', price=1)
+
+    with raises(ValueError):
+        Traversal(
+            muckefuck, 'a_name', {
+                'node_class': Supplier, 'direction': INCOMING,
+                'relationship_type': 'KNOWS', 'model': None
+            }
+        )
+
+    Traversal(
+        muckefuck, 'a_name', {
+            'node_class': Supplier, 'direction': INCOMING,
+            'relation_type': 'KNOWS', 'model': None
+        }
+    )
+
+
+def test_empty_filters():
+    """Test this case:
+        ```
+            SomeModel.nodes.filter().filter(Q(arg1=val1)).all()
+            SomeModel.nodes.exclude().exclude(Q(arg1=val1)).all()
+            SomeModel.nodes.filter().filter(arg1=val1).all()
+       ```
+       In django_rest_framework filter uses such as lazy function and
+       ``get_queryset`` function in ``GenericAPIView`` should returns
+       ``NodeSet`` object.
+    """
+
+    for c in Coffee.nodes:
+        c.delete()
+
+    c1 = Coffee(name="Super", price=5, id_=1).save()
+    c2 = Coffee(name="Puper", price=10, id_=2).save()
+
+    empty_filter = Coffee.nodes.filter()
+
+    all_coffees = empty_filter.all()
+    assert len(all_coffees) == 2, "unexpected number of results"
+
+    filter_empty_filter = empty_filter.filter(price=5)
+    assert len(filter_empty_filter.all()) == 1, "unexpected number of results"
+    assert c1 in filter_empty_filter.all(), "doesnt contain c1 in ``filter_empty_filter``"
+
+    filter_q_empty_filter = empty_filter.filter(Q(price=5))
+    assert len(filter_empty_filter.all()) == 1, "unexpected number of results"
+    assert c1 in filter_empty_filter.all(), "doesnt contain c1 in ``filter_empty_filter``"
+
+
+def test_q_filters():
+
+    for c in Coffee.nodes:
+        c.delete()
+
+    c1 = Coffee(name="Icelands finest", price=5, id_=1).save()
+    c2 = Coffee(name="Britains finest", price=10, id_=2).save()
+    c3 = Coffee(name="Japans finest", price=35, id_=3).save()
+    c4 = Coffee(name="US extra-fine", price=None, id_=4).save()
+    c5 = Coffee(name="Latte", price=35, id_=5).save()
+    c6 = Coffee(name="Cappuccino", price=35, id_=6).save()
+
+    coffees_5_10 = Coffee.nodes.filter(Q(price=10) | Q(price=5)).all()
+    assert len(coffees_5_10) == 2, "unexpected number of results"
+    assert c1 in coffees_5_10, "doesnt contain 5 price coffee"
+    assert c2 in coffees_5_10, "doesnt contain 10 price coffee"
+
+    coffees_5_6 = Coffee.nodes.filter(Q(name="Latte") | Q(name="Cappuccino")).filter(price=35).all()
+    assert len(coffees_5_6) == 2, "unexpected number of results"
+    assert c5 in coffees_5_6, "doesnt contain 5 coffee"
+    assert c6 in coffees_5_6, "doesnt contain 6 coffee"
+
+    coffees_5_6 = Coffee.nodes.filter(price=35).filter(Q(name="Latte") | Q(name="Cappuccino")).all()
+    assert len(coffees_5_6) == 2, "unexpected number of results"
+    assert c5 in coffees_5_6, "doesnt contain 5 coffee"
+    assert c6 in coffees_5_6, "doesnt contain 6 coffee"
+
+    finest_coffees = Coffee.nodes.filter(name__iendswith=' Finest').all()
+    assert len(finest_coffees) == 3, "unexpected number of results"
+    assert c1 in finest_coffees, "doesnt contain 1st finest coffee"
+    assert c2 in finest_coffees, "doesnt contain 2nd finest coffee"
+    assert c3 in finest_coffees, "doesnt contain 3rd finest coffee"
+
+    unpriced_coffees = Coffee.nodes.filter(Q(price__isnull=True)).all()
+    assert len(unpriced_coffees) == 1, "unexpected number of results"
+    assert c4 in unpriced_coffees, "doesnt contain unpriced coffee"
+
+    coffees_with_id_gte_3 = Coffee.nodes.filter(Q(id___gte=3)).all()
+    assert len(coffees_with_id_gte_3) == 4, "unexpected number of results"
+    assert c3 in coffees_with_id_gte_3
+    assert c4 in coffees_with_id_gte_3
+    assert c5 in coffees_with_id_gte_3
+    assert c6 in coffees_with_id_gte_3
+
+    coffees_5_not_japans = Coffee.nodes.filter(Q(price__gt=5) & ~Q(name="Japans finest")).all()
+    assert c3 not in coffees_5_not_japans
+
+
+def test_traversal_filter_left_hand_statement():
+    nescafe = Coffee(name='Nescafe2', price=99).save()
+    nescafe_gold = Coffee(name='Nescafe gold', price=11).save()
+
+    tesco = Supplier(name='Sainsburys', delivery_cost=3).save()
+    biedronka = Supplier(name='Biedronka', delivery_cost=5).save()
+    lidl = Supplier(name='Lidl', delivery_cost=3).save()
+
+    nescafe.suppliers.connect(tesco)
+    nescafe_gold.suppliers.connect(biedronka)
+    nescafe_gold.suppliers.connect(lidl)
+
+    lidl_supplier = NodeSet(Coffee.nodes.filter(price=11).suppliers).filter(delivery_cost=3).all()
+
+    assert lidl in lidl_supplier
\ No newline at end of file
